@@ -1,7 +1,4 @@
-context("test-mold-formulas")
-
 test_that("can mold simple formulas", {
-
   sparse_bp <- default_formula_blueprint(composition = "dgCMatrix")
   matrix_bp <- default_formula_blueprint(composition = "matrix")
 
@@ -9,24 +6,23 @@ test_that("can mold simple formulas", {
   x2 <- mold(fac_1 ~ num_1, example_train, blueprint = sparse_bp)
   x3 <- mold(fac_1 ~ num_1, example_train, blueprint = matrix_bp)
 
-  expect_is(x1$predictors, "tbl_df")
-  expect_is(x2$predictors, "dgCMatrix")
-  expect_is(x3$predictors, "matrix")
+  expect_s3_class(x1$predictors, "tbl_df")
+  expect_s4_class(x2$predictors, "dgCMatrix")
+  expect_matrix(x3$predictors)
 
   expect_equal(colnames(x1$predictors), "num_1")
   expect_equal(colnames(x2$predictors), "num_1")
   expect_equal(colnames(x3$predictors), "num_1")
-  expect_is(x1$outcomes, "tbl_df")
-  expect_is(x2$outcomes, "tbl_df")
-  expect_is(x3$outcomes, "tbl_df")
+  expect_s3_class(x1$outcomes, "tbl_df")
+  expect_s3_class(x2$outcomes, "tbl_df")
+  expect_s3_class(x3$outcomes, "tbl_df")
   expect_equal(colnames(x1$outcomes), "fac_1")
   expect_equal(x1$outcomes, x2$outcomes)
   expect_equal(x1$outcomes, x3$outcomes)
-  expect_is(x1$blueprint, "default_formula_blueprint")
+  expect_s3_class(x1$blueprint, "default_formula_blueprint")
 })
 
 test_that("can mold multivariate formulas", {
-
   sparse_bp <- default_formula_blueprint(composition = "dgCMatrix")
   matrix_bp <- default_formula_blueprint(composition = "matrix")
 
@@ -34,7 +30,7 @@ test_that("can mold multivariate formulas", {
   x2 <- mold(num_1 + num_2 ~ num_3, example_train, blueprint = sparse_bp)
   x3 <- mold(num_1 + num_2 ~ num_3, example_train, blueprint = matrix_bp)
 
-  expect_is(x1$outcomes, "tbl_df")
+  expect_s3_class(x1$outcomes, "tbl_df")
   expect_equal(colnames(x1$outcomes), c("num_1", "num_2"))
   expect_equal(x1$outcomes, x2$outcomes)
   expect_equal(x1$outcomes, x3$outcomes)
@@ -53,17 +49,15 @@ test_that("can mold multivariate formulas", {
   )
   expect_equal(y1$outcomes, y2$outcomes)
   expect_equal(y1$outcomes, y3$outcomes)
-
 })
 
 test_that("factor predictors with no intercept are fully expanded", {
-
-  x1  <- mold(
+  x1 <- mold(
     num_1 ~ fac_1,
     example_train,
     blueprint = default_formula_blueprint(intercept = TRUE)
   )
-  x2  <- mold(
+  x2 <- mold(
     num_1 ~ fac_1,
     example_train,
     blueprint = default_formula_blueprint(intercept = TRUE, composition = "matrix")
@@ -77,9 +71,11 @@ test_that("factor predictors with no intercept are fully expanded", {
   y2 <- mold(
     num_1 ~ fac_1,
     example_train,
-    blueprint = default_formula_blueprint(intercept = FALSE,
-                                          indicators = "one_hot",
-                                          composition = "matrix")
+    blueprint = default_formula_blueprint(
+      intercept = FALSE,
+      indicators = "one_hot",
+      composition = "matrix"
+    )
   )
 
   expect_equal(
@@ -99,11 +95,9 @@ test_that("factor predictors with no intercept are fully expanded", {
     colnames(y2$predictors),
     c("fac_1a", "fac_1b", "fac_1c")
   )
-
 })
 
 test_that("can mold and not expand dummies", {
-
   x <- mold(
     num_1 ~ fac_1,
     example_train,
@@ -111,111 +105,125 @@ test_that("can mold and not expand dummies", {
   )
 
   expect_equal(colnames(x$predictors), "fac_1")
-  expect_is(x$predictors$fac_1, "factor")
+  expect_s3_class(x$predictors$fac_1, "factor")
   expect_equal(x$blueprint$indicators, "none")
 })
 
 test_that("errors are thrown if `indicator = 'none'` and factor interactions exist", {
-
   expect_error(
-    mold(~ fac_1, example_train, blueprint = default_formula_blueprint(indicators = "none")),
+    mold(~fac_1, example_train, blueprint = default_formula_blueprint(indicators = "none")),
     NA
   )
 
-  expect_error(
+  expect_snapshot(error = TRUE, {
     mold(
       num_1 ~ fac_1:num_2,
       example_train,
       blueprint = default_formula_blueprint(indicators = "none")
-    ),
-    "Interaction terms involving factors"
-  )
+    )
+  })
 
   # Checking various types of generated interactions
 
-  expect_error(
+  expect_snapshot(error = TRUE, {
     mold(
       num_1 ~ fac_1:num_2,
       example_train,
       blueprint = default_formula_blueprint(indicators = "none")
-    ),
-    "'fac_1'"
-  )
-
-  expect_error(
+    )
+  })
+  expect_snapshot(error = TRUE, {
     mold(
       num_1 ~ fac_1 * num_2,
       example_train,
       blueprint = default_formula_blueprint(indicators = "none")
-    ),
-    "'fac_1'"
-  )
-
-  expect_error(
+    )
+  })
+  expect_snapshot(error = TRUE, {
     mold(
-      num_1 ~ (fac_1 + num_2) ^ 2,
+      num_1 ~ (fac_1 + num_2)^2,
       example_train,
       blueprint = default_formula_blueprint(indicators = "none")
-    ),
-    "'fac_1'"
-  )
-
-  expect_error(
+    )
+  })
+  expect_snapshot(error = TRUE, {
     mold(
       num_1 ~ fac_1 %in% num_2,
       example_train,
       blueprint = default_formula_blueprint(indicators = "none")
-    ),
-    "'fac_1'"
-  )
-
-  # Both factor issues are reported
+    )
+  })
 
   example_train2 <- example_train
   example_train2$fac_12 <- example_train2$fac_1
 
-  expect_error(
+  expect_snapshot(error = TRUE, {
     mold(
       ~ fac_1:fac_12,
       example_train2,
       blueprint = default_formula_blueprint(indicators = "none")
-    ),
-    "'fac_1', 'fac_12'."
-  )
-
+    )
+  })
 })
 
 test_that("errors are thrown if `indicator = 'none'` and factors are used in inline functions", {
-
   blueprint_no_indicators <- default_formula_blueprint(indicators = "none")
 
-  expect_error(
-    mold(~ paste0(fac_1), example_train, blueprint = blueprint_no_indicators),
-    "Functions involving factors"
-  )
-
-  expect_error(
-    mold(~ paste0(fac_1), example_train, blueprint = blueprint_no_indicators),
-    "'fac_1'"
-  )
-
-  expect_error(
+  expect_snapshot(error = TRUE, {
+    mold(~ paste0(fac_1), example_train, blueprint = blueprint_no_indicators)
+  })
+  expect_snapshot(error = TRUE, {
+    mold(~ paste0(fac_1), example_train, blueprint = blueprint_no_indicators)
+  })
+  expect_snapshot(error = TRUE, {
     mold(~ fac_1 %>% paste0(), example_train, blueprint = blueprint_no_indicators)
-  )
-
-  expect_error(
-    mold(~ paste0(fac_1 + fac_1), example_train, blueprint = blueprint_no_indicators),
-    "'fac_1'"
-  )
+  })
+  expect_snapshot(error = TRUE, {
+    mold(~ paste0(fac_1 + fac_1), example_train, blueprint = blueprint_no_indicators)
+  })
+  expect_snapshot(error = TRUE, {
+    mold(~ (fac_1) & num_1, example_train, blueprint = blueprint_no_indicators)
+  })
+  expect_snapshot(error = TRUE, {
+    mold(~ (fac_1 & num_1), example_train, blueprint = blueprint_no_indicators)
+  })
 
   example_train2 <- example_train
   example_train2$fac_12 <- example_train2$fac_1
 
-  expect_error(
-    mold(~ paste0(fac_1) + paste0(fac_12), example_train2, blueprint = blueprint_no_indicators),
-    "'fac_1', 'fac_12'."
-  )
+  expect_snapshot(error = TRUE, {
+    mold(~ paste0(fac_1) + paste0(fac_12), example_train2, blueprint = blueprint_no_indicators)
+  })
+})
 
+test_that("`indicators = 'none'` doesn't error for allowed inline functions", {
+  df <- tibble(y = 1:2, x = factor(c("a", "b")), x2 = c(2, 3))
+
+  blueprint_no_indicators <- default_formula_blueprint(indicators = "none")
+
+  out <- mold(y ~ (x), df, blueprint = blueprint_no_indicators)
+  expect_identical(out$predictors$x, df$x)
+
+  out <- mold(y ~ (x) + x, df, blueprint = blueprint_no_indicators)
+  expect_identical(out$predictors$x, df$x)
+
+  out <- mold(y ~ (x) - x2, df, blueprint = blueprint_no_indicators)
+  expect_identical(out$predictors, df["x"])
+
+  out <- mold(y ~ 1 + x2 + x, df, blueprint = blueprint_no_indicators)
+  expect_identical(out$predictors$x, df$x)
+
+  out <- mold(y ~ 1 - x2 + (x), df, blueprint = blueprint_no_indicators)
+  expect_identical(out$predictors$x, df$x)
+})
+
+test_that("`indicators = 'none'` doesn't error for names with spaces in them (#217)", {
+  df <- vctrs::data_frame(y = 1:2, `foo bar` = factor(c("a", "b")))
+
+  blueprint_no_indicators <- default_formula_blueprint(indicators = "none")
+
+  out <- mold(y ~ `foo bar`, df, blueprint = blueprint_no_indicators)
+  expect_identical(out$predictors[["foo bar"]], df[["foo bar"]])
 })
 
 test_that("`indicators = 'none'` doesn't error if a non-factor name regex-matches a factor name (#182)", {
@@ -241,9 +249,8 @@ test_that("`indicators = 'none'` doesn't error if an inline function regex-match
 })
 
 test_that("`indicators = 'none'` works fine in strange formulas", {
-
   x <- mold(
-    ~ NULL,
+    ~NULL,
     example_train,
     blueprint = default_formula_blueprint(indicators = "none", intercept = TRUE)
   )
@@ -252,11 +259,9 @@ test_that("`indicators = 'none'` works fine in strange formulas", {
     colnames(x$predictors),
     "(Intercept)"
   )
-
 })
 
 test_that("formula intercepts can be added", {
-
   x1 <- mold(
     fac_1 ~ num_1,
     example_train,
@@ -279,7 +284,6 @@ test_that("formula intercepts can be added", {
 })
 
 test_that("can mold formulas with special terms", {
-
   bp <- default_formula_blueprint(composition = "dgCMatrix")
   x1 <- mold(fac_1 ~ num_1:num_2 + I(num_1^2), example_train)
   x2 <- mold(fac_1 ~ num_1:num_2 + I(num_1^2), example_train, blueprint = bp)
@@ -337,7 +341,6 @@ test_that("global environment variables cannot be used", {
 })
 
 test_that("cannot manually remove intercept in the formula itself", {
-
   bp <- default_formula_blueprint(composition = "dgCMatrix")
   expect_error(
     mold(fac_1 ~ y + 0, example_train),
@@ -357,46 +360,35 @@ test_that("cannot manually remove intercept in the formula itself", {
     mold(fac_1 ~ y - 1, example_train),
     "`formula` must not contain"
   )
-
 })
 
 test_that("RHS with _only_ intercept related terms are caught", {
-
   bp <- default_formula_blueprint(composition = "dgCMatrix")
 
-  expect_error(
-    mold(~ 0, example_train),
-    "`formula` must not contain the intercept removal term"
-  )
-  expect_error(
-    mold(~ 0, example_train, blueprint = bp),
-    "`formula` must not contain the intercept removal term"
-  )
-
-  expect_error(
-    mold(~ 1, example_train),
-    "`formula` must not contain the intercept term"
-  )
-
-  expect_error(
-    mold(~ -1, example_train),
-    "`formula` must not contain the intercept removal term"
-  )
+  expect_snapshot(error = TRUE, {
+    mold(~0, example_train)
+  })
+  expect_snapshot(error = TRUE, {
+    mold(~0, example_train, blueprint = bp)
+  })
 
+  expect_snapshot(error = TRUE, {
+    mold(~1, example_train)
+  })
+  expect_snapshot(error = TRUE, {
+    mold(~ -1, example_train)
+  })
 })
 
 test_that("`NULL` can be used to represent empty RHS formulas", {
-
   bp <- default_formula_blueprint(composition = "dgCMatrix")
 
-  expect_error(
-    mold(~ 0, example_train),
-    "`formula` must not contain the intercept removal term"
-  )
-  expect_error(
-    mold(~ 0, example_train, blueprint = bp),
-    "`formula` must not contain the intercept removal term"
-  )
+  expect_snapshot(error = TRUE, {
+    mold(~0, example_train)
+  })
+  expect_snapshot(error = TRUE, {
+    mold(~0, example_train, blueprint = bp)
+  })
 
   expect_error(
     x1 <- mold(~NULL, example_train),
@@ -418,11 +410,9 @@ test_that("`NULL` can be used to represent empty RHS formulas", {
   )
 
   expect_equal(colnames(y$predictors), "(Intercept)")
-
 })
 
 test_that("intercepts can still be added when not using indicators (i.e. model.matrix())", {
-
   x <- mold(
     num_2 ~ fac_1,
     example_train,
@@ -433,15 +423,13 @@ test_that("intercepts can still be added when not using indicators (i.e. model.m
     "(Intercept)" %in% colnames(x$predictors)
   )
 
-  expect_is(
+  expect_s3_class(
     x$predictors$fac_1,
     "factor"
   )
-
 })
 
 test_that("`data` is validated", {
-
   bp <- default_formula_blueprint(composition = "dgCMatrix")
   expect_error(
     mold(fac_1 ~ num_2, 1),
@@ -451,78 +439,68 @@ test_that("`data` is validated", {
     mold(fac_1 ~ num_2, 1, blueprint = bp),
     "`data` must be a data.frame or a matrix"
   )
-
 })
 
 test_that("full interaction syntax is supported", {
-
   expect_equal(
-    mold(~ fac_1*num_2, example_train)$predictors,
+    mold(~ fac_1 * num_2, example_train)$predictors,
     mold(~ fac_1 + num_2 + fac_1:num_2, example_train)$predictors
   )
 
   expect_equal(
-    mold(~ fac_1*num_2 - fac_1:num_2, example_train)$predictors,
+    mold(~ fac_1 * num_2 - fac_1:num_2, example_train)$predictors,
     mold(~ fac_1 + num_2, example_train)$predictors
   )
 
   expect_equal(
-    mold(~ (num_2 + num_1 + num_3) ^ 2, example_train)$predictors,
-    mold(~ num_2 +
-           num_1 +
-           num_3 +
-           num_2:num_1 +
-           num_2:num_3 +
-           num_1:num_3,
-         example_train)$predictors
+    mold(~ (num_2 + num_1 + num_3)^2, example_train)$predictors,
+    mold(
+      ~ num_2 +
+        num_1 +
+        num_3 +
+        num_2:num_1 +
+        num_2:num_3 +
+        num_1:num_3,
+      example_train
+    )$predictors
   )
 
   expect_equal(
     mold(~ num_2 + num_1 %in% num_2, example_train)$predictors,
     mold(~ num_2 + num_2:num_1, example_train)$predictors
   )
-
 })
 
 test_that("`indicators = 'none'` runs numeric interactions", {
-
   x <- mold(~ num_1:num_2, example_train,
-            blueprint = default_formula_blueprint(indicators = "none"))
+    blueprint = default_formula_blueprint(indicators = "none")
+  )
 
   expect_equal(
     colnames(x$predictors),
     "num_1:num_2"
   )
-
 })
 
 test_that("LHS of the formula cannot contain interactions", {
-
-  expect_error(
-    mold(num_1:num_2 ~ num_2, example_train),
-    "The following interaction terms were found: 'num_1:num_2'"
-  )
-
-  expect_error(
-    mold(num_1*num_2 ~ num_2, example_train),
-    "The following interaction terms were found: 'num_1:num_2'"
-  )
-
-  expect_error(
-    mold(num_1 %in% num_2 ~ num_2, example_train),
-    "The following interaction terms were found: 'num_1:num_2'"
-  )
-
-  expect_error(
-    mold((num_1 + num_2)^2 ~ num_2, example_train),
-    "The following interaction terms were found: 'num_1:num_2'"
-  )
-
-  expect_error(
-    mold(num_1:num_2 + fac_1:num_1 ~ num_2, example_train),
-    "The following interaction terms were found: 'num_1:num_2', 'num_1:fac_1'"
-  )
-
+  expect_snapshot(error = TRUE, {
+    mold(num_1:num_2 ~ num_2, example_train)
+  })
+  expect_snapshot(error = TRUE, {
+    mold(num_1 * num_2 ~ num_2, example_train)
+  })
+  expect_snapshot(error = TRUE, {
+    mold(num_1 %in% num_2 ~ num_2, example_train)
+  })
+  expect_snapshot(error = TRUE, {
+    mold((num_1 + num_2)^2 ~ num_2, example_train)
+  })
+  expect_snapshot(error = TRUE, {
+    mold(num_1:num_2 + fac_1:num_1 ~ num_2, example_train)
+  })
+  expect_snapshot(error = TRUE, {
+    mold(num_1 / num_2 ~ num_2, example_train)
+  })
 })
 
 test_that("LHS of the formula won't misinterpret `::` as an interaction (#174)", {
@@ -531,7 +509,6 @@ test_that("LHS of the formula won't misinterpret `::` as an interaction (#174)",
 })
 
 test_that("original predictor and outcome classes are recorded", {
-
   bp <- default_formula_blueprint(composition = "dgCMatrix")
   x1 <- mold(log(num_1) ~ log(num_2), example_train)
   x2 <- mold(log(num_1) ~ log(num_2), example_train, blueprint = bp)
@@ -553,11 +530,9 @@ test_that("original predictor and outcome classes are recorded", {
     get_data_classes(x2$blueprint$ptypes$outcomes),
     list(num_1 = "integer")
   )
-
 })
 
 test_that("`.` notation works as expected", {
-
   bp <- default_formula_blueprint(composition = "dgCMatrix")
   x1 <- mold(fac_1 ~ ., example_train)
   x2 <- mold(fac_1 ~ ., example_train, blueprint = bp)
@@ -581,7 +556,6 @@ test_that("`.` notation works as expected", {
     colnames(x2$blueprint$ptypes$outcomes),
     "fac_1"
   )
-
 })
 
 # `expand_formula_dot_notation()` does not expand LHS dots, and we check
@@ -600,7 +574,6 @@ test_that("`.` notation fails on the LHS", {
 })
 
 test_that("`.` notation with variable as predictor and outcome", {
-
   bp <- default_formula_blueprint(composition = "dgCMatrix")
   x1 <- mold(num_2 ~ . + num_2, example_train)
   x2 <- mold(num_2 ~ . + num_2, example_train)
@@ -643,15 +616,13 @@ test_that("`.` notation with variable as predictor and outcome", {
     colnames(y2$blueprint$ptypes$outcomes),
     "num_2"
   )
-
 })
 
 test_that("`.` notation with no outcome works fine", {
-
   bp <- default_formula_blueprint(composition = "dgCMatrix")
   # Uses all columns of example_train
-  x1 <- mold(~ ., example_train)
-  x2 <- mold(~ ., example_train, blueprint = bp)
+  x1 <- mold(~., example_train)
+  x2 <- mold(~., example_train, blueprint = bp)
 
   expect_equal(
     ncol(x1$predictors),
@@ -670,7 +641,6 @@ test_that("`.` notation with no outcome works fine", {
     colnames(x2$blueprint$ptypes$predictors),
     c("num_1", "num_2", "num_3", "fac_1", "fac_2")
   )
-
 })
 
 test_that("`-var` still registers var as a predictor", {
@@ -682,14 +652,13 @@ test_that("`-var` still registers var as a predictor", {
   expect_true(
     "num_1" %in% colnames(x$blueprint$ptypes$predictors)
   )
-
 })
 
 test_that("Missing y value returns a 0 column tibble for `outcomes`", {
   bp <- default_formula_blueprint(composition = "dgCMatrix")
-  x1 <- mold(~ num_2, example_train)
+  x1 <- mold(~num_2, example_train)
   x2 <- mold(NULL ~ num_2, example_train)
-  x3 <- mold(~ num_2, example_train, blueprint = bp)
+  x3 <- mold(~num_2, example_train, blueprint = bp)
   x4 <- mold(NULL ~ num_2, example_train, blueprint = bp)
 
   expect_equal(nrow(x1$outcomes), 12)
@@ -703,21 +672,29 @@ test_that("Missing y value returns a 0 column tibble for `outcomes`", {
 
 test_that("Missing y value returns a 0 column / 0 row tibble for `ptype`", {
   bp <- default_formula_blueprint(composition = "dgCMatrix")
-  x1 <- mold(~ num_2, example_train)
-  x2 <- mold(~ num_2, example_train, blueprint = bp)
+  x1 <- mold(~num_2, example_train)
+  x2 <- mold(~num_2, example_train, blueprint = bp)
   expect_equal(x1$blueprint$ptypes$outcomes, tibble())
   expect_equal(x2$blueprint$ptypes$outcomes, tibble())
 })
 
 test_that("Missing y value still has outcome `terms` present", {
-  x <- mold(~ num_2, example_train)
+  x <- mold(~num_2, example_train)
 
   expect_equal(
-    rlang::f_rhs(x$blueprint$terms$outcomes),
-    rlang::expr(NULL + 0)
+    f_rhs(x$blueprint$terms$outcomes),
+    expr(NULL + 0)
   )
 })
 
+test_that("`blueprint` is validated", {
+  df <- tibble(x = 1)
+
+  expect_snapshot(error = TRUE, {
+    mold(~x, df, blueprint = 1)
+  })
+})
+
 # ------------------------------------------------------------------------------
 # Character predictors
 
@@ -782,6 +759,69 @@ test_that("character predictors are left as characters when `indicators` is 'non
   expect_true(is.character(x$blueprint$ptypes$predictors$z))
 })
 
+test_that("character vectors with `indicators = traditional/one_hot` store levels in `levels` (#213)", {
+  df <- tibble(x = c("a", "b", "c"), y = factor(c("d", "e", "e")), z = c("g", "f", "g"))
+
+  bp <- default_formula_blueprint(indicators = "traditional")
+  x <- mold(~x + y + z, df, blueprint = bp)
+
+  # Only from character columns, and the levels get sorted
+  # (like in base R's `model.matrix()` and `prep(strings_as_factors = TRUE)`)
+  expect_identical(
+    x$blueprint$levels,
+    list(
+      x = c("a", "b", "c"),
+      z = c("f", "g")
+    )
+  )
+
+  # We leave the `ptype` untouched, mirroring the original data
+  expect_identical(x$blueprint$ptypes$predictors$x, character())
+  expect_identical(x$blueprint$ptypes$predictors$y, vec_ptype(df$y))
+  expect_identical(x$blueprint$ptypes$predictors$z, character())
+
+  bp <- default_formula_blueprint(indicators = "one_hot")
+  x <- mold(~x + y + z, df, blueprint = bp)
+
+  # Only from character columns, and the levels get sorted
+  # (like in base R's `model.matrix()` and `prep(strings_as_factors = TRUE)`)
+  expect_identical(
+    x$blueprint$levels,
+    list(
+      x = c("a", "b", "c"),
+      z = c("f", "g")
+    )
+  )
+})
+
+test_that("character vectors with `indicators = none` don't use `levels` (#213)", {
+  df <- tibble(x = c("a", "b", "c"), y = factor(c("d", "e", "e")), z = c("g", "f", "g"))
+
+  bp <- default_formula_blueprint(indicators = "none")
+  x <- mold(~x + y + z, df, blueprint = bp)
+
+  expect_identical(x$blueprint$levels, list())
+
+  expect_identical(x$blueprint$ptypes$predictors$x, character())
+  expect_identical(x$blueprint$ptypes$predictors$y, vec_ptype(df$y))
+  expect_identical(x$blueprint$ptypes$predictors$z, character())
+})
+
+test_that("character vectors with `indicators = none` works with constant columns (#213)", {
+  df <- tibble(x = "a", y = factor("d"), z = "g")
+
+  bp <- default_formula_blueprint(indicators = "none")
+  x <- mold(~x + y + z, df, blueprint = bp)
+
+  expect_identical(x$blueprint$ptypes$predictors$x, character())
+  expect_identical(x$blueprint$ptypes$predictors$y, vec_ptype(df$y))
+  expect_identical(x$blueprint$ptypes$predictors$z, character())
+
+  expect_identical(x$predictors$x, df$x)
+  expect_identical(x$predictors$y, df$y)
+  expect_identical(x$predictors$z, df$z)
+})
+
 # ------------------------------------------------------------------------------
 # Factor encodings
 
