@@ -6,3 +6,11 @@ test_that("an outcome can also be a predictor and is only returned once", {
     "Sepal.Length"
   )
 })
+
+test_that("`data` must be data-like", {
+  ptype <- data.frame(x = integer())
+
+  expect_snapshot(error = TRUE, {
+    shrink(1, ptype)
+  })
+})
